#include "FrameBuffer.hpp"

#include <iostream>

#include <GL/glew.h>

#include "RenderBuffer.hpp"

#include "../Utils/GLUtils.hpp"

FrameBuffer :: FrameBuffer()
{
    glGenFramebuffers(1, &fid);
    std::cout << "New FrameBuffer id: " << fid << std::endl;
    if ( fid == 0 )
    {
        throw FrameBufferException("FrameBuffer id is 0");
    }

    GLUtils::checkError("glGenFrameBuffers");
    std::cout << "FrameBuffer created" << std::endl;
}

FrameBuffer :: ~FrameBuffer()
{
    glDeleteFramebuffers(1, &fid);
    std::cout << "FrameBuffer '" << fid << "' deleted" << std::endl;
}

void FrameBuffer :: bind(const GLenum target)
{
    glBindFramebuffer(target,fid);
    GLUtils::checkError("glBindFrameBuffer");
}

void FrameBuffer :: unbind(const GLenum target)
{
    glBindFramebuffer(target,0);
    GLUtils::checkError("glBindFrameBuffer (unbind)");
}


void FrameBuffer :: attach(const GLenum target, const GLenum attachmentPoint, const GLuint textureId)
{
    this->bind(target);
    glFramebufferTexture2D(target, attachmentPoint, GL_TEXTURE_2D, textureId, 0);
    GLUtils::checkError("glFramebufferTexture2D");
    this->unbind(target);
}

void FrameBuffer :: attach(const GLenum target, const GLenum attachmentPoint, const RenderBuffer& renderBuffer)
{
    this->bind(target);
    glFramebufferRenderbuffer(target, attachmentPoint, GL_RENDERBUFFER, renderBuffer.getRID());
    GLUtils::checkError("glFramebufferRenderbuffer");
    this->unbind(target);
}

void FrameBuffer :: validate(const GLenum target)
{
    GLenum result = glCheckFramebufferStatus(target);
    if ( result != GL_FRAMEBUFFER_COMPLETE )
    {
        std::cerr << "Fail to validate the FBO" << std::endl;
    }
}
