#include "LineRenderer.hpp"

#include <GL/glew.h>

#include "../Types/Vector3.hpp"
#include "../Types/Vector4.hpp"

void LineRenderer :: addLine(const Point& p1, const Point& p2)
{
    int actualSize = vertices.size();

    vertices.push_back(p1);
    vertices.push_back(p2);

    indices.push_back(actualSize);
    indices.push_back(actualSize+1);
}

void LineRenderer :: addLines(const std::vector<Point>& listPoints)
{
    std::vector<Point>::iterator itEnd = vertices.end();
    int actualSize = vertices.size();
    int sizeAdded = listPoints.size();

    if ( listPoints.size() % 2 == 0 )
    {
        vertices.insert(itEnd, listPoints.begin(), listPoints.end());
    }
    else
    {
        std::cout << "Warning: You are trying to add a odd number to the LineRenderer (last point truncated)" << std::endl;
        vertices.insert(itEnd, listPoints.begin(), listPoints.end()-1);
    }

    for (int i = 0 ; i < sizeAdded ; i++ )
    {
        indices.push_back(actualSize+i);
    }
}

void LineRenderer :: addLines(const std::vector<Point>& listPoints, const std::vector<unsigned int> listIndices)
{
    unsigned int actualIndicesNumber = vertices.size();
    for ( std::vector<Point>::const_iterator itPoint = listPoints.begin() ; itPoint != listPoints.end() ; ++itPoint )
    {
        vertices.push_back(*itPoint);
    }

    for ( std::vector<unsigned int>::const_iterator itIndice = listIndices.begin() ; itIndice != listIndices.end() ; ++itIndice )
    {
        indices.push_back(*itIndice + actualIndicesNumber);
    }
}

void LineRenderer :: draw(unsigned int lineSize)
{
    glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_COLOR_ARRAY);

    // Assign our data to OpenGL
    glVertexPointer(3, GL_FLOAT, sizeof(Vector3f) + sizeof(Colour4), &vertices[0].position.x);
    glColorPointer(4, GL_FLOAT, sizeof(Vector3f) + sizeof(Colour4), &vertices[0].colour.r);

    glLineWidth(lineSize);
    glDrawElements(GL_LINES,indices.size(), GL_UNSIGNED_INT, &indices[0]);

    glDisableClientState(GL_COLOR_ARRAY);
    glDisableClientState(GL_VERTEX_ARRAY);
}
