#ifndef __SHADER_HPP__
#define __SHADER_HPP__

#include <exception>
#include <string>

#include <GL/glew.h>

class ShaderException : public std::exception
{
    private:
        std::string message;

    public:
        ShaderException(const std::string& message):message(message) {}
        virtual ~ShaderException()throw() {}

        virtual const char* what() const throw()
        {
            return message.c_str();
        }
};

class ShaderCreationException : public ShaderException
{
    public:
        ShaderCreationException():ShaderException("Fail to create the shader") {}
        virtual ~ShaderCreationException()throw() {}
};

class ShaderLoadingException : public ShaderException
{
    public:
        ShaderLoadingException(const std::string& shaderFileName):ShaderException(std::string("Fail to create the shader: ") + shaderFileName) {}
        virtual ~ShaderLoadingException()throw() {}
};

class ShaderUniformNotFoundException : public ShaderException
{
    public:
        ShaderUniformNotFoundException(const std::string& uniformName):ShaderException(std::string("Fail to find the uniform: ") + uniformName) {}
        virtual ~ShaderUniformNotFoundException()throw() {}
};

class Shader
{
    private:
        GLint vsid;
        GLint psid;
        GLint pgid;

        unsigned int getFileSize(FILE* const pFile);
        char* readFile(const std::string& fileName);

        char checkShaderCompilation(const std::string& shaderName, GLuint shaderID);

    public:

        Shader(const std::string& vertexShaderFileName, const std::string& pixelShaderFile);
        ~Shader();

        void enable();
        static void disable();
        GLint getProgram();
};

#endif
