#include "Texture.hpp"

#include <cassert>

#include <GL/glew.h>

#include "../Types/Vector4.hpp"

#include "../Utils/GLUtils.hpp"

Texture :: Texture(const std::string& fileName):tid(0)
{
    assert(false);
}

Texture :: Texture(const unsigned int width, const unsigned height):tid(0)
{
    createID();
    this->bind();

    setParameters();

    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);

    this->unbind();

    GLUtils::checkError("Texture creation");
}

Texture :: Texture(const unsigned int width, const unsigned int height, const Colour4& c):tid(0)
{
    createID();
    this->bind();

    char* pPixels = new char[width * height * 4];
    for ( unsigned int i = 0 ; i < width * height ; i++ )
    {
        pPixels[i*4] = c.r * 255;
        pPixels[i*4+1] = c.g * 255;
        pPixels[i*4+2] = c.b * 255;
        pPixels[i*4+3] = c.a * 255;
    }

    setParameters();

    glTexImage2D(GL_TEXTURE_2D, 0, 4, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, pPixels);
    GLUtils::checkError("glTexImage2D");

    this->unbind();

    delete [] pPixels;

    // GLUtils::checkError("Texture creation (with colour)");
}

Texture :: ~Texture()
{
    glDeleteTextures(1,&tid);
}

void Texture :: createID()
{
    glGenTextures(1,&tid);
    GLUtils::checkError("glGenTextures");
    std::cout << "GL Texture ID created: " << tid << std::endl;
}

void Texture :: setParameters()
{
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    GLUtils::checkError("glTexParameterf");
}

void Texture :: bind()const
{
    glBindTexture(GL_TEXTURE_2D,tid);
    GLUtils::checkError("glBindTexture");
}

void Texture :: unbind()
{
    glBindTexture(GL_TEXTURE_2D,0);
    GLUtils::checkError("glBindTexture (unbind)");
}


