#ifndef __GRID_HPP__
#define __GRID_HPP__

#include "../3D/LineRenderer.hpp"

#include "../Types/Vector4.hpp"

class Grid
{
    private:

        unsigned int numberPointsX;
        unsigned int numberPointsY;

        std::vector < Point > points;
        std::vector < unsigned int > indices;

        std::vector < unsigned int > hadImpulsion; // 0 no impulsion ; 1 doing impulsion ; 2 impulsed
        unsigned int impulsionUpdateStep;

        std::vector<Colour4> colours;

        std::vector < std::vector < unsigned int > > picsSpectrum;

        float impulsionValue;

    public:

        Grid(unsigned int numberPointsX, unsigned int numberPointsY, float step);

        const std::vector<Point>& getPoints() { return points; }
        const std::vector<unsigned int>& getIndices() { return indices; }

        void makeElevation(unsigned int x, unsigned int y, float height, int displacement=0);
        void makeImpulsion(unsigned int x, unsigned int y, const Colour4& colour, unsigned int impulsionStep=1, unsigned int limit=-1);

        void updateSpectrum(float* spectrum, const unsigned int precision);
        void updateImpulsion();

        unsigned int getNumberPointsX() { return numberPointsX; }
        unsigned int getNumberPointsY() { return numberPointsY; }

        void clearColour();
        void clearColour(const Colour3& colour);
        void clearElevation();

        float getHeight(unsigned int x, unsigned int y) { return points[x + y * numberPointsX].position.y; }

        void setImpulsionValue(float newImpulsionValue) { impulsionValue = newImpulsionValue; }
};

#endif
