#include "Pyramide.hpp"

#include <vector>

#include <GL/glew.h>

#include "../3D/LineRenderer.hpp"

#include "../Types/Vector3.hpp"

Pyramide :: Pyramide(Vector3f position, Vector3f rotation, Colour3 colour, float scale/*=1*/):position(position),rotation(rotation),active(false)
{
    std::vector<Point> points;

    float min = -scale;
    float max = scale;

    points.push_back(Point(Vector3f(min,min,min),
                           colour));
    points.push_back(Point(Vector3f(max,min,min),
                           colour));
    points.push_back(Point(Vector3f(min,min,max),
                           colour));
    points.push_back(Point(Vector3f(max,min,max),
                           colour));

    points.push_back(Point(Vector3f(0,max*1.5f,0),
                           colour));

    std::vector<unsigned int> indices;

    indices.push_back(0);
    indices.push_back(1);
    indices.push_back(0);
    indices.push_back(2);
    indices.push_back(0);
    indices.push_back(4);

    indices.push_back(1);
    indices.push_back(3);
    indices.push_back(1);
    indices.push_back(4);

    indices.push_back(2);
    indices.push_back(4);
    indices.push_back(2);
    indices.push_back(3);

    indices.push_back(3);
    indices.push_back(4);

    renderer.addLines(points,indices);
}


void Pyramide::draw()
{
    glPushMatrix();

    glTranslatef(position.x, position.y, position.z);

    glRotatef(rotation.x, 1,0,0);
    glRotatef(rotation.y, 0,1,0);
    glRotatef(rotation.z, 0,0,1);

    renderer.draw();

    glPopMatrix();
}

void Pyramide :: update()
{
    if ( active )
    {
        position.y += 0.05f;
    }
}
