#include "Sphere.hpp"

#include <vector>

#include <GL/glew.h>

#include <cmath>

#include "../3D/LineRenderer.hpp"

#include "../Types/Vector3.hpp"

#include "../Utils/MathUtils.hpp"

Sphere :: Sphere(Vector3f position, Vector3f rotation, Colour3 colour, float size, unsigned int nbPpL, float offset):position(position),rotation(rotation),offset(offset),active(false)
{
    float nbPointsPerLine = nbPpL;
    float stepLine = 360.0f/nbPpL;
    float stepHigh = 180.0f/nbPpL;
    float nbPoints = nbPointsPerLine * nbPpL;
    int indexPoint = 0;

    std::vector<Point> vertices;
    std::vector<unsigned int> indices;

    vertices.resize(nbPoints * 4);
    indices.reserve(nbPoints * 4);

    for ( float theta = -90 ; theta < 90 ; theta += stepHigh )
    {
        for ( float phi = 0 ; phi < 360 ; phi += stepLine )
        {
            float rTheta = MathUtils::degToRan(theta);
            float rTheta2 = MathUtils::degToRan(theta+stepHigh);
            float rPhi = MathUtils::degToRan(phi);
            float rPhi2 = MathUtils::degToRan(phi+stepLine);

             vertices[indexPoint+3] = Point(Vector3f(cos(rTheta) * cos(rPhi) * size,
                                                     cos(rTheta) * sin(rPhi)  * size,
                                                     sin(rTheta) * size),
                                            colour);

            vertices[indexPoint] = Point(Vector3f(cos(rTheta2) * cos(rPhi) * size,
                                                  cos(rTheta2) * sin(rPhi)  * size,
                                                  sin(rTheta2) * size),
                                         colour);

            vertices[indexPoint+1] = Point(Vector3f(cos(rTheta2) * cos(rPhi2) * size,
                                                    cos(rTheta2) * sin(rPhi2)  * size,
                                                    sin(rTheta2) * size),
                                        colour);

            vertices[indexPoint+2] = Point(Vector3f(cos(rTheta) * cos(rPhi2) * size,
                                                    cos(rTheta) * sin(rPhi2)  * size,
                                                    sin(rTheta) * size),
                                           colour);

            //writefln("Normal %d: %f;%f;%f",indexPoint,normals[indexPoint].x,normals[indexPoint].y,normals[indexPoint].z);
            indexPoint+=4;
        }
    }

    indexPoint = 0;
    for ( float theta = -90 ; theta <= 90-stepHigh ; theta += stepHigh )
    {
        for ( float phi = 0 ; phi < 360 ; phi += stepLine )
        {/*
            if ( phi != 360-step )
            {
                indices[indexPoint] = (phi / step) + ((theta+90) / step) * nbPointsPerLine;
                indices[indexPoint+1] = ((phi+step) / step) + ((theta+90) / step) * nbPointsPerLine;
                indices[indexPoint+2] = ((phi+step) / step) + (((theta+step)+90) / step) * nbPointsPerLine;
                indices[indexPoint+3] = (phi / step) + (((theta+step)+90) / step) * nbPointsPerLine;
            }
            else
            {
                indices[indexPoint] = (phi / step) + ((theta+90) / step) * nbPointsPerLine;
                indices[indexPoint+1] = 0 + ((theta+90) / step) * nbPointsPerLine;
                indices[indexPoint+2] = 0 + (((theta+step)+90) / step) * nbPointsPerLine;
                indices[indexPoint+3] = (phi / step) + (((theta+step)+90) / step) * nbPointsPerLine;
            }
*/
            indices[indexPoint]=0;
            indices[indexPoint+1]=0;
            indices[indexPoint+2]=0;
            indices[indexPoint+3]=0;
            indexPoint+=4;
        }
    }

    renderer.addLines(vertices);
}

void Sphere::draw()
{
    glPushMatrix();

    glTranslatef(position.x, position.y, position.z);

    glRotatef(rotation.x, 1,0,0);
    glRotatef(rotation.y, 0,1,0);
    glRotatef(rotation.z, 0,0,1);

    renderer.draw();

    glPopMatrix();
}

void Sphere :: update()
{
    if ( active )
    {
        position.y += 0.05f;
    }
}
