#include "ScenePartWorld.hpp"

#include <stdlib.h>

#include <GL/gl.h>

#include "../NEngine/Window.hpp"

#include "../3D/Camera/Camera.hpp"
#include "../3D/LineRenderer.hpp"

#include "../Objects/Grid.hpp"
#include "../Objects/Particules5.hpp"
#include "../Objects/Particules4.hpp"

#include "../Types/Vector3.hpp"
#include "../Types/Vector4.hpp"

#include "../Music/Music.hpp"

#include "../Utils/MathUtils.hpp"

ScenePartWorld :: ScenePartWorld():currentCam(0),startTime(0)
{
    coloursList.reserve(6);

    coloursList[0] = Colour4(0.0,0.74,0.19,1.0);
    coloursList[1] = Colour4(0.39,0.87,0.52,1.0);
    coloursList[2] = Colour4(0.19,0.086,0.72,1.0);
    coloursList[3] = Colour4(0.39,0.29,0.847,1.0);
    coloursList[4] = Colour4(1.0,0.8,0.0,1.0);
    coloursList[5] = Colour4(1.0,0.11,0.0,1.0);

    cams.push_back(new LinearCamera(Vector3f(20,5.5,20),Vector3f(75,0,75),Vector3f(30,1,30),Vector3f(78,1,78),Vector3f(0,1,0), 25000));

    parts.push_back(new Particules4(Vector3f(45,3,45),Colour4(0.0,0.74,0.19,1.0),500,0.2));
    parts.push_back(new Particules4(Vector3f(45,3-0.010,45),Colour4(0.39,0.87,0.52,1.0),500,0.4));
    parts.push_back(new Particules4(Vector3f(45,3-0.02,45),Colour4(0.19,0.086,0.72,1.0),500,0.6));
    parts.push_back(new Particules4(Vector3f(45,3-0.03,45),Colour4(0.39,0.29,0.847,1.0),500,0.7));
    parts.push_back(new Particules4(Vector3f(45,3-0.04,45),Colour4(1.0,0.8,0.0,1.0),500,0.9));
    parts.push_back(new Particules4(Vector3f(45,3-0.05,45),Colour4(1.0,0.11,0.0,1.0),500,1.1));

    parts2.push_back(new Particules5(Vector3f(55,2,45),Colour4(0.0,0.74,0.19,1.0),500,0.2));
    parts2.push_back(new Particules5(Vector3f(55,2-0.010,45),Colour4(0.39,0.87,0.52,1.0),500,0.4));
    parts2.push_back(new Particules5(Vector3f(55,2-0.02,45),Colour4(0.19,0.086,0.72,1.0),500,0.6));
    parts2.push_back(new Particules5(Vector3f(55,2-0.03,45),Colour4(0.39,0.29,0.847,1.0),500,0.7));
    parts2.push_back(new Particules5(Vector3f(55,2-0.04,45),Colour4(1.0,0.8,0.0,1.0),500,0.9));
    parts2.push_back(new Particules5(Vector3f(55,2-0.05,45),Colour4(1.0,0.11,0.0,1.0),500,1.1));

    parts2.push_back(new Particules5(Vector3f(45,2,60),Colour4(0.0,0.74,0.19,1.0),500,0.2));
    parts2.push_back(new Particules5(Vector3f(45,2-0.010,60),Colour4(0.39,0.87,0.52,1.0),500,0.4));
    parts2.push_back(new Particules5(Vector3f(45,2-0.02,60),Colour4(0.19,0.086,0.72,1.0),500,0.6));
    parts2.push_back(new Particules5(Vector3f(45,2-0.03,60),Colour4(0.39,0.29,0.847,1.0),500,0.7));
    parts2.push_back(new Particules5(Vector3f(45,2-0.04,60),Colour4(1.0,0.8,0.0,1.0),500,0.9));
    parts2.push_back(new Particules5(Vector3f(45,2-0.05,60),Colour4(1.0,0.11,0.0,1.0),500,1.1));

    for ( unsigned int i = 0 ; i < 150 ; i++ )
    {
        float x = rand()/ (float)RAND_MAX * 150;
        float y = 0;/*rand()/ (float)RAND_MAX * 30 - 50;*/
        float z = rand()/ (float)RAND_MAX * 150;

        cubes.push_back(new Cube(Vector3f(x,y,z), Vector3f(0,0,0), Colour3(117.0f/255.0f,117.0f/255.0f,117.0f/255.0f), 0.5));
    }

    for ( unsigned int i = 0 ; i < 150 ; i++ )
    {
        float x = rand()/ (float)RAND_MAX * 150;
        float y = 0;/*rand()/ (float)RAND_MAX * 30 - 40;*/
        float z = rand()/ (float)RAND_MAX * 150;

        piramides.push_back(new Pyramide(Vector3f(x,y,z), Vector3f(0,0,0), Colour3(117.0f/255.0f,117.0f/255.0f,117.0f/255.0f), 0.5));
    }

    for ( unsigned int i = 0 ; i < 150 ; i++ )
    {
        float x = rand()/ (float)RAND_MAX * 150;
        float y = 0;/*rand()/ (float)RAND_MAX * 30 - 60;*/
        float z = rand()/ (float)RAND_MAX * 150;

        spheres.push_back(new Sphere(Vector3f(x,y,z), Vector3f(0,0,0), Colour3(117.0f/255.0f,117.0f/255.0f,117.0f/255.0f), 0.5, 10));
    }
}

ScenePartWorld :: ~ScenePartWorld()
{
    for ( std::vector<Cube*>::iterator itCube = cubes.begin() ; itCube != cubes.end() ; ++itCube )
    {
        delete (*itCube);
    }

    for ( std::vector<Pyramide*>::iterator itPiramide = piramides.begin() ; itPiramide != piramides.end() ; ++itPiramide )
    {
        delete (*itPiramide);
    }

    for ( std::vector<Sphere*>::iterator itSphere = spheres.begin() ; itSphere != spheres.end() ; ++itSphere )
    {
        delete (*itSphere);
    }

    for ( std::vector<Particules4*>::iterator itParticule = parts.begin() ; itParticule != parts.end() ; ++itParticule )
    {
        delete (*itParticule);
    }

    for ( std::vector<Particules5*>::iterator itParticule = parts2.begin() ; itParticule != parts2.end() ; ++itParticule )
    {
        delete (*itParticule);
    }

    for ( std::vector<LinearCamera*>::iterator itPLC = cams.begin() ; itPLC != cams.end() ; ++itPLC )
    {
        delete (*itPLC);
    }
}

void ScenePartWorld :: launch()
{
    for ( std::vector<Cube*>::iterator itCube = cubes.begin() ; itCube != cubes.end() ; ++itCube )
    {
        if ( (*itCube)->getActive() == false )
        {
            (*itCube)->setActive(true);
            break;
        }
    }

    for ( std::vector<Pyramide*>::iterator itPiramide = piramides.begin() ; itPiramide != piramides.end() ; ++itPiramide )
    {
        if ( (*itPiramide)->getActive() == false )
        {
            (*itPiramide)->setActive(true);
            break;
        }
    }

    for ( std::vector<Sphere*>::iterator itSphere = spheres.begin() ; itSphere != spheres.end() ; ++itSphere )
    {
        if ( (*itSphere)->getActive() == false )
        {
            (*itSphere)->setActive(true);
            break;
        }
    }
}

void ScenePartWorld :: setSceneRender()
{
    cams[currentCam]->apply();
}

void ScenePartWorld :: draw(Window* pWindow)
{
    for ( std::vector<Cube*>::iterator itCube = cubes.begin() ; itCube != cubes.end() ; ++itCube )
    {
        (*itCube)->draw();
    }

    for ( std::vector<Pyramide*>::iterator itPiramide = piramides.begin() ; itPiramide != piramides.end() ; ++itPiramide )
    {
        (*itPiramide)->draw();
    }

    for ( std::vector<Sphere*>::iterator itSphere = spheres.begin() ; itSphere != spheres.end() ; ++itSphere )
    {
        (*itSphere)->draw();
    }


    glDisable(GL_DEPTH_TEST);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA,GL_ONE);

    for ( std::vector<Particules5*>::iterator itParts = parts2.begin() ; itParts != parts2.end() ; ++itParts )
    {
        (*itParts)->draw();
    }

    for ( std::vector<Particules4*>::iterator itParts = parts.begin() ; itParts != parts.end() ; ++itParts )
    {
        (*itParts)->draw();
    }

    glDisable(GL_BLEND);
    glEnable(GL_DEPTH_TEST);
}

bool ScenePartWorld :: update(const Camera& camera, Grid* pGrid, Music* pMusic, unsigned int time)
{
    pGrid->setImpulsionValue(8);

    for ( std::vector<Cube*>::iterator itCube = cubes.begin() ; itCube != cubes.end() ; ++itCube )
    {
        (*itCube)->update();
    }

    for ( std::vector<Pyramide*>::iterator itPiramide = piramides.begin() ; itPiramide != piramides.end() ; ++itPiramide )
    {
        (*itPiramide)->update();
    }

    for ( std::vector<Sphere*>::iterator itSphere = spheres.begin() ; itSphere != spheres.end() ; ++itSphere )
    {
        (*itSphere)->update();
    }

    for ( std::vector<Particules5*>::iterator itParts = parts2.begin() ; itParts != parts2.end() ; ++itParts )
    {
        (*itParts)->update();
    }

    for ( std::vector<Particules4*>::iterator itParts = parts.begin() ; itParts != parts.end() ; ++itParts )
    {
        (*itParts)->update();
    }

    if ( startTime == 0 )
    {
        startTime = time;
    }

    fadeValue = 1;

    bool isNextCamera = cams[currentCam]->update(time);
    if ( isNextCamera == false )
    {
        startTime += cams[currentCam]->getMsTime();

        currentCam++;
        if ( currentCam >= cams.size() )
        {
            fadeValue = 0;
            return false;
        }
    }

    unsigned int camTime = (time - startTime) % cams[currentCam]->getMsTime();
    if ( camTime < 500 )
    {
        fadeValue = camTime / 500.0f;
    }
    else if ( camTime > (cams[currentCam]->getMsTime() - 500) )
    {
        fadeValue = (cams[currentCam]->getMsTime() - camTime) / 500.0f;
    }

    static float spectrum[512] = { 0 };
    pMusic->getSpectrum(spectrum,512);

{
    bool started = false;

    for ( unsigned int i = 0 ; i < 10 ; i++ )
    {
        if ( spectrum[i] > 0.2f )
        {
            unsigned colour = rand()%6;

            unsigned int yImp = (rand()/(float)RAND_MAX) * pGrid->getNumberPointsX()-10;
            unsigned int xImp = (rand()/(float)RAND_MAX) * pGrid->getNumberPointsY()-10;

            pGrid->makeImpulsion(xImp, yImp ,Colour4(0,0,0,1), 1, 6);

            if ( started == false )
            {
                launch();
                started = true;
            }
        }
    }
}

    pGrid->updateImpulsion();

    if ( pulseValue > 0.0 )
    {
        pulseValue -= 0.08f;
    }
    else
    {
        pulseValue = 0;
    }

    if ( spectrum[2] >= 0.25f )
    {
        pulseValue = 1;
    }
/*
    if  ( time - startTime > 15000 )
    {
        pGrid->clearColour(Colour3(0,0,0));
    }
    else if ( time - startTime > 10000 )
    {
        pGrid->clearColour(MathUtils::lerp(Colour3(0,0,0), Colour3(117.0f/255.0f,117.0f/255.0f,117.0f/255.0f),(15000-(time - startTime))/3000.0f));
    }
*/
    if ( time - startTime > 10000 )
    {
        fadeValue = (25000 - (time - startTime)) / 15000.0f;
    }

    return true;
}
