#include "SceneSadSpectrum.hpp"

#include <GL/gl.h>

#include "../NEngine/Window.hpp"
#include "../3D/Camera/Camera.hpp"
#include "../3D/LineRenderer.hpp"
#include "../Objects/Grid.hpp"
#include "../Music/Music.hpp"

SceneSadSpectrum :: SceneSadSpectrum():currentCam(0),f(Colour4(0.9,0.9,0.9,0.7)),startTime(0),step(0)
{
    cams.push_back(new LinearCamera(Vector3f(92,4,56),Vector3f(40,5.5,48),Vector3f(86,1,52),Vector3f(34,1,48),Vector3f(0,1,0), 5500));
    cams.push_back(new LinearCamera(Vector3f(52,5.5,48),Vector3f(104,6.5,96),Vector3f(52,1,67),Vector3f(12,0,67),Vector3f(0,1,0), 5500));
    cams.push_back(new LinearCamera(Vector3f(48,8,48),Vector3f(96,8,48),Vector3f(38,0,38),Vector3f(28,0,38),Vector3f(0,1,0), 5500));
    cams.push_back(new LinearCamera(Vector3f(56,4,104),Vector3f(12,5.5,47),Vector3f(23,1,56),Vector3f(46,1,29),Vector3f(0,1,0), 5532));

    // cams.push_back(new LinearCamera(Vector3f(36,5.5,24),Vector3f(49,4,59),Vector3f(12,1,34),Vector3f(98,1,89),Vector3f(0,1,0), 5000));
    // cams.push_back(new LinearCamera(Vector3f(49,5.5,59),Vector3f(79,4,89),Vector3f(98,1,89),Vector3f(37,1,43),Vector3f(0,1,0), 10000));
}

SceneSadSpectrum :: ~SceneSadSpectrum()
{
    for ( std::vector<LinearCamera*>::iterator itPLC = cams.begin() ; itPLC != cams.end() ; ++itPLC )
    {
        delete (*itPLC);
    }
}

void SceneSadSpectrum :: setSceneRender()
{
    cams[currentCam]->apply();
}

void SceneSadSpectrum :: draw(Window* pWindow)
{
}

bool SceneSadSpectrum :: update(const Camera& camera, Grid* pGrid, Music* pMusic, unsigned int time)
{
    if ( startTime == 0 )
    {
        startTime = time;
        return true;
    }

    bool isNextCamera = cams[currentCam]->update(time);
    if ( isNextCamera == false )
    {
        currentCam++;
        if ( currentCam >= cams.size() )
        {
            fadeValue = 0;
            return false;
        }
    }

    unsigned int camTime = (time - startTime) % 5000;
    if ( camTime < 500 )
    {
        fadeValue = camTime / 500.0f;
    }
    else if ( camTime > (5000 - 500) )
    {
        fadeValue = (5000 - camTime) / 500.0f;
    }

    static float spectrum[512] = { 0 };
    pMusic->getSpectrum(spectrum,512);

    pGrid->updateSpectrum(spectrum,512);

    return true;
}
