#include "SceneSpectrumImpulsePart.hpp"

#include <stdlib.h>

#include <GL/gl.h>

#include "../NEngine/Window.hpp"

#include "../3D/Camera/Camera.hpp"
#include "../3D/LineRenderer.hpp"

#include "../Objects/Grid.hpp"
#include "../Objects/Particules3.hpp"

#include "../Types/Vector4.hpp"

#include "../Music/Music.hpp"

SceneSpectrumImpulsePart :: SceneSpectrumImpulsePart():currentCam(0),startTime(0)
{
    cams.push_back(new LinearCamera(Vector3f(25,5.5,20),Vector3f(65,2,20),Vector3f(35,1,30),Vector3f(75,1,30),Vector3f(0,1,0), 15000));

    parts.push_back(new Particules3(Vector3f(55,3,40),Colour4(0.0,0.74,0.19,1.0),500,0.2));
    parts.push_back(new Particules3(Vector3f(55,3-0.010,40),Colour4(0.39,0.87,0.52,1.0),500,0.4));
    parts.push_back(new Particules3(Vector3f(55,3-0.02,40),Colour4(0.19,0.086,0.72,1.0),500,0.6));
    parts.push_back(new Particules3(Vector3f(55,3-0.03,40),Colour4(0.39,0.29,0.847,1.0),500,0.7));
    parts.push_back(new Particules3(Vector3f(55,3-0.04,40),Colour4(1.0,0.8,0.0,1.0),500,0.9));
    parts.push_back(new Particules3(Vector3f(55,3-0.05,40),Colour4(1.0,0.11,0.0,1.0),500,1.1));
}

SceneSpectrumImpulsePart :: ~SceneSpectrumImpulsePart()
{
    for ( std::vector<LinearCamera*>::iterator itPLC = cams.begin() ; itPLC != cams.end() ; ++itPLC )
    {
        delete (*itPLC);
    }
}

void SceneSpectrumImpulsePart :: setSceneRender()
{
    cams[currentCam]->apply();
}

void SceneSpectrumImpulsePart :: draw(Window* pWindow)
{
    glDisable(GL_DEPTH_TEST);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA,GL_ONE);

    for ( std::vector<Particules3*>::iterator itParts = parts.begin() ; itParts != parts.end() ; ++itParts )
    {
        (*itParts)->draw();
    }

    glDisable(GL_BLEND);
    glEnable(GL_DEPTH_TEST);
}

bool SceneSpectrumImpulsePart :: update(const Camera& camera, Grid* pGrid, Music* pMusic, unsigned int time)
{
    pGrid->setImpulsionValue(8);

    for ( std::vector<Particules3*>::iterator itParts = parts.begin() ; itParts != parts.end() ; ++itParts )
    {
        (*itParts)->update();
    }

    if ( startTime == 0 )
    {
        startTime = time;
    }

    fadeValue = 1;

    bool isNextCamera = cams[currentCam]->update(time);
    if ( isNextCamera == false )
    {
        startTime += cams[currentCam]->getMsTime();

        currentCam++;
        if ( currentCam >= cams.size() )
        {
            fadeValue = 0;
            return false;
        }
    }

    unsigned int camTime = (time - startTime) % cams[currentCam]->getMsTime();
    if ( camTime < 500 )
    {
        fadeValue = camTime / 500.0f;
    }
    else if ( camTime > (cams[currentCam]->getMsTime() - 500) )
    {
        fadeValue = (cams[currentCam]->getMsTime() - camTime) / 500.0f;
    }

    static float spectrum[512] = { 0 };
    static float fakeSpectrum[512] = { 0 };
    pMusic->getSpectrum(spectrum,512);
    pGrid->updateSpectrum(spectrum,512);

    for ( unsigned int i = 0 ; i < 8 ; i++ )
    {
        if ( spectrum[i] > 0.2f )
        {
            float r = rand()/(float)RAND_MAX;
            float g = rand()/(float)RAND_MAX;
            float b = rand()/(float)RAND_MAX;
            unsigned int yImp = (rand()/(float)RAND_MAX) * 20 + (cams[currentCam]->getPosition().x/1.5f);
            unsigned int xImp = (rand()/(float)RAND_MAX) * 10 + 20;

            pGrid->makeImpulsion(xImp, yImp ,Colour4(r,g,b,0.7), 1, 6);
        }
    }

    //std::cout << "Camera: " << cams[currentCam]->getPosition() <<  std::endl;

    pGrid->updateImpulsion();

    if ( pulseValue > 0.0 )
    {
        pulseValue -= 0.04f;
    }
    else
    {
        pulseValue = 0;
    }

    if ( spectrum[2] >= 0.25f )
    {
        pulseValue = 1;
    }

    return true;
}
