#include "SceneSpectrumPart.hpp"

#include <stdlib.h>

#include <GL/gl.h>

#include "../NEngine/Window.hpp"

#include "../3D/Camera/Camera.hpp"
#include "../3D/LineRenderer.hpp"

#include "../Objects/Grid.hpp"
#include "../Objects/Particules2.hpp"

#include "../Types/Vector4.hpp"

#include "../Music/Music.hpp"

SceneSpectrumPart :: SceneSpectrumPart():currentCam(0),startTime(0)
{
    cams.push_back(new LinearCamera(Vector3f(49,5.5,59),Vector3f(79,4,89),Vector3f(98,1,89),Vector3f(37,1,43),Vector3f(0,1,0), 5000));
    cams.push_back(new LinearCamera(Vector3f(79,5.5,89),Vector3f(40,4,40),Vector3f(40,1,50),Vector3f(20,1,20),Vector3f(0,1,0), 5000));

    parts.push_back(new Particules2(Vector3f(20,1,20),Colour4(0.0,0.74,0.19,1.0),500,0.2));
    parts.push_back(new Particules2(Vector3f(20,1-0.010,20),Colour4(0.39,0.87,0.52,1.0),500,0.4));
    parts.push_back(new Particules2(Vector3f(20,1-0.02,20),Colour4(0.19,0.086,0.72,1.0),500,0.6));
    parts.push_back(new Particules2(Vector3f(20,1-0.03,20),Colour4(0.39,0.29,0.847,1.0),500,0.7));
    parts.push_back(new Particules2(Vector3f(20,1-0.04,20),Colour4(1.0,0.8,0.0,1.0),500,0.9));
    parts.push_back(new Particules2(Vector3f(20,1-0.05,20),Colour4(1.0,0.11,0.0,1.0),500,1.1));

    parts.push_back(new Particules2(Vector3f(10,2,10),Colour4(0.0,0.74,0.19,1.0),500,1.2));
    parts.push_back(new Particules2(Vector3f(10,2-0.010,10),Colour4(0.39,0.87,0.52,1.0),500,1.4));
    parts.push_back(new Particules2(Vector3f(10,2-0.02,10),Colour4(0.19,0.086,0.72,1.0),500,1.6));
    parts.push_back(new Particules2(Vector3f(10,2-0.03,10),Colour4(0.39,0.29,0.847,1.0),500,1.7));
    parts.push_back(new Particules2(Vector3f(10,2-0.04,10),Colour4(1.0,0.8,0.0,1.0),500,1.9));
    parts.push_back(new Particules2(Vector3f(10,2-0.05,10),Colour4(1.0,0.11,0.0,1.0),500,2.1));
}

SceneSpectrumPart :: ~SceneSpectrumPart()
{
    for ( std::vector<LinearCamera*>::iterator itPLC = cams.begin() ; itPLC != cams.end() ; ++itPLC )
    {
        delete (*itPLC);
    }
}

void SceneSpectrumPart :: setSceneRender()
{
    cams[currentCam]->apply();
}

void SceneSpectrumPart :: draw(Window* pWindow)
{
    glDisable(GL_DEPTH_TEST);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA,GL_ONE);

    for ( std::vector<Particules2*>::iterator itParts = parts.begin() ; itParts != parts.end() ; ++itParts )
    {
        (*itParts)->draw();
    }

    glDisable(GL_BLEND);
    glEnable(GL_DEPTH_TEST);
}

bool SceneSpectrumPart :: update(const Camera& camera, Grid* pGrid, Music* pMusic, unsigned int time)
{
    pGrid->setImpulsionValue(10);

    for ( std::vector<Particules2*>::iterator itParts = parts.begin() ; itParts != parts.end() ; ++itParts )
    {
        (*itParts)->update();
    }

    if ( startTime == 0 )
    {
        startTime = time;
        return true;
    }

    bool isNextCamera = cams[currentCam]->update(time);
    if ( isNextCamera == false )
    {
        startTime += cams[currentCam]->getMsTime();

        currentCam++;
        if ( currentCam >= cams.size() )
        {
            fadeValue = 0;
            return false;
        }
    }

    unsigned int camTime = (time - startTime) % cams[currentCam]->getMsTime();
    if ( camTime < 500 )
    {
        fadeValue = camTime / 500.0f;
    }
    else if ( camTime > (cams[currentCam]->getMsTime() - 500) )
    {
        fadeValue = (cams[currentCam]->getMsTime() - camTime) / 500.0f;
    }

    static float spectrum[512] = { 0 };
    pMusic->getSpectrum(spectrum,512);

    pGrid->updateSpectrum(spectrum,512);

    if ( pulseValue > 0.0 )
    {
        pulseValue -= 0.04f;
    }
    else
    {
        pulseValue = 0;
    }

    if ( spectrum[2] >= 0.25f )
    {
        pulseValue = 1;
    }

    return true;
}
