#ifndef __VECTOR3_HPP__
#define __VECTOR3_HPP__

#include <iostream>

template <typename T>
struct Vector3
{
    union
    {
        T x;
        T r;
    };

    union
    {
        T y;
        T g;
    };

    union
    {
        T z;
        T b;
    };

    Vector3():x(0),y(0),z(0) {}
    Vector3(const T& x, const T& y, const T& z):x(x),y(y),z(z) {}

    void normalise()
    {
        float norm = sqrt(this.x * this.x + this.y * this.y + this.z * this.z);

        if ( norm != 0 )
        {
            this.x /= norm;
            this.y /= norm;
            this.z /= norm;
        }
    }

    Vector3<T> cross(const Vector3<T>& v)
    {
        return Vector3<T>(v.y * v.z - v.z * v.y,
                          v.z * v.x - v.x * v.z,
                          v.x * v.y - v.y * v.x);
    }

    float dot(const Vector3<T>& v)
    {
        return this.x * v.x + this.y * v.y + this.z * v.z;
    }

    Vector3<T> operator+(const Vector3<T>& v)
    {
        return Vector3<T>(v.x + this->x,
                          v.y + this->y,
                          v.z + this->z);
    }

    Vector3<T> operator-(const Vector3<T>& v)
    {
        return Vector3<T>(this->x - v.x,
                          this->y - v.y,
                          this->z - v.z);
    }

    Vector3<T> operator*(float value)
    {
        return Vector3<T>(this->x * value,
                          this->y * value,
                          this->z * value);
    }

    template <typename U>
    friend std::ostream& operator<<(std::ostream& str, const Vector3<U>& v);
};

template <typename T>
Vector3<T> operator+(const Vector3<T>& v1, const Vector3<T>& v2)
{
    return Vector3<T>(v1.x + v2.x,
                      v1.y + v2.y,
                      v1.z + v2.z);
}

template <typename T>
Vector3<T> operator-(const Vector3<T>& v1, const Vector3<T>& v2)
{
    return Vector3<T>(v1.x - v2.x,
                      v1.y - v2.y,
                      v1.z - v2.z);
}

template <typename T>
Vector3<T> operator*(const Vector3<T>& v, float value)
{
    return Vector3<T>(v.x * value,
                      v.y * value,
                      v.z * value);
}

template <typename U>
std::ostream& operator<<(std::ostream& str, const Vector3<U>& v)
{
    str << "X: " << v.x << " Y: " << v.y << " Z: " << v.z;
    return str;
}

typedef Vector3<float> Vector3f;
typedef Vector3<int> Vector3i;
typedef Vector3<float> Colour3;

#endif
