#ifndef __GLUTILS_HPP__
#define __GLUTILS_HPP__

#include <string>
#include <exception>

#include <GL/glew.h>

class GLErrorException : public std::exception
{
    private:

        std::string message;

    public:

        GLErrorException(GLenum errorId, const std::string& functionName)
        {
            switch (errorId)
            {
                case GL_NO_ERROR:
                    message = "No error occured (so why this exception has been lunched";
                    break;

                case GL_INVALID_ENUM:
                    message = "Invalid enumerator";
                    break;

                case GL_INVALID_VALUE:
                    message = "Invalid value";
                    break;

                case GL_INVALID_OPERATION:
                    message = "Invalid operation";
                    break;

                case GL_OUT_OF_MEMORY:
                    message = "Out of memory";
                    break;

                default:
                    message = "Unknown error";
                    break;
            }

            message +=  std::string(" in '") + functionName + std::string("'");
        }

        virtual const char* what() const throw()
        {
            return message.c_str();
        }

        virtual ~GLErrorException()throw() {}
};

class GLUtils
{
    public:

        static void checkError(const std::string& functionName)
        {
            GLenum errorType = glGetError();
            if ( errorType != GL_NO_ERROR )
            {
                throw GLErrorException(errorType, functionName);
            }
        }
};

#endif
