#include <iostream>

#include <GL/glew.h>
// #include <SDL/SDL.h>

#include "NEngine/Engine.hpp"
#include "NEngine/Window.hpp"
#include "NEngine/Controllers/InputManager.hpp"
#include "NEngine/Controllers/SDL_Keyboard.hpp"

#include "3D/Camera/TopDownCamera.hpp"
#include "3D/Camera/RotatingCamera.hpp"
#include "3D/Shader.hpp"
#include "3D/Texture.hpp"
#include "3D/RenderBuffer.hpp"
#include "3D/FrameBuffer.hpp"
#include "3D/LineRenderer.hpp"
#include "3D/ScreenQuad.hpp"

#include "Math/Bezier.hpp"

#include "Objects/Grid.hpp"
#include "Objects/Star.hpp"
#include "Objects/Cube.hpp"
#include "Objects/Pyramide.hpp"
#include "Objects/Sphere.hpp"
#include "Objects/Particules.hpp"
#include "Objects/Font.hpp"
#include "Objects/Scroller.hpp"

#include "Scenes/SceneManager.hpp"
#include "Scenes/SceneTitle.hpp"
#include "Scenes/SceneSad.hpp"
#include "Scenes/SceneSadSpectrum.hpp"
#include "Scenes/SceneSadSpectrumGreet.hpp"
#include "Scenes/SceneSpectrumPart.hpp"
#include "Scenes/SceneSpectrumImpulsePart.hpp"
#include "Scenes/ScenePartWorld.hpp"

#include "Music/Music.hpp"
#include "Music/Impulsor.hpp"

#include "Utils/GLUtils.hpp"
#include "Utils/MathUtils.hpp"
#include "Utils/Settings.hpp"

#include "Types/Vector3.hpp"
#include "Types/Vector4.hpp"

#include <cmath>
#include <cstdlib>

// const unsigned int R2T_TEXTURE_SIZE = 2048;

int main(int argc, char** argv)
{
    float spectrum[512];

    try
    {
        Settings setting("data/settings.ini");
        srand(setting.getSetting("seed",42));
        Engine e(EAPI_SDL | EAPI_OpenGL, setting.getSetting("hres",640),setting.getSetting("vres",480),"DemoD");

        if ( e.checkExtension("GL_ARB_framebuffer_object") == false || e.checkExtension("GL_ARB_framebuffer_sRGB") == false )
        {
            std::cout << "The demo needs better hardware. Sorry about it" << std::endl;
            return -0;
        }

        Music m;
        m.create();
        m.loadFile("data/music/flawed.mp3");

        InputManager im;

        im.registerController(new SDL_Keyboard());
/*
        lr.addLine(Point(Vector3f(0,0,0), Colour4(1.0,0,0,0)),
                   Point(Vector3f(1,0,0), Colour4(1.0,0,0,0)));
        lr.addLine(Point(Vector3f(0,0,0), Colour4(0,1,0,0)),
                   Point(Vector3f(0,1,0), Colour4(0,1,0,0)));
        lr.addLine(Point(Vector3f(0,0,0), Colour4(0,0,1,0)),
                   Point(Vector3f(0,0,1), Colour4(0,0,1,0)));
*/

        RotatingCamera c(Vector3f(6,0,6), 6, 0.3f);
        GLenum glError = glGetError();
        if ( glError != GL_NO_ERROR )
        {
            std::cerr << "OpenGL Error: " << glError << std::endl;
        }

        std::cout << "Scene creation" << std::endl;
        SceneManager sm;

        // sm.addScene(new SceneTitle());
        SceneSad* pSceneSad = new SceneSad();
        SceneSadSpectrum* pSceneSadSpectrum = new SceneSadSpectrum();
        sm.addScene(pSceneSad);
        sm.addScene(pSceneSadSpectrum);
        sm.addScene(new SceneSadSpectrumGreet());
        sm.addScene(new SceneSpectrumPart());
        sm.addScene(new SceneSpectrumImpulsePart());
        sm.addScene(new ScenePartWorld());

        std::cout << "Geometry generation" << std::endl;

        // Star s(Vector3f(0,3,0),Vector3f(0,0,0),Colour3(0.6,0.0,0.0));
        // Cube cube(Vector3f(2,0,0),Vector3f(0,0,0),Colour3(0.0,0.0,0.6),0.1f);
        // Pyramide pyramid(Vector3f(2,0,0),Vector3f(0,0,0),Colour3(0.0,0.6,0.6),0.2f);
        // Sphere sphere(Vector3f(0,1,1),Vector3f(0,0,0),Colour3(0.6,0.3,0.8),0.3f,10);

        Shader* pSimpleShader = NULL;
        Shader* pHBlurShader = NULL;
        Shader* pVBlurShader = NULL;

        try
        {
            pSimpleShader = new Shader("data/shaders/pulse.vs","data/shaders/pulse.ps");
            pHBlurShader = new Shader("data/shaders/hblur.vs","data/shaders/hblur.ps");
            pVBlurShader = new Shader("data/shaders/vblur.vs","data/shaders/vblur.ps");
        }
        catch (ShaderException se )
        {
            std::cout << se.what() << std::endl;
            return -3;
        }

        std::cout << "Texture preparation (R2T)" << std::endl;
        glEnable(GL_TEXTURE_2D);
        glDisable(GL_CULL_FACE);

        Texture textureBucket(e.getWindow()->getWidth(),e.getWindow()->getHeight());
        Texture textureHBlurBucket(e.getWindow()->getWidth(),e.getWindow()->getHeight());
/*
        std::cout << "Render buffer preparation (R2T)" << std::endl;
        RenderBuffer renderBuffer;
        renderBuffer.create(GL_RENDERBUFFER, GL_RGBA,e.getWindow()->getWidth(),e.getWindow()->getHeight());
*/
        std::cout << "Frame buffer preparation (R2T)" << std::endl;
        FrameBuffer frameBuffer;
        frameBuffer.attach(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, textureBucket.getID());
        //frameBuffer.attach(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, renderBuffer);
        frameBuffer.bind(GL_FRAMEBUFFER);
        frameBuffer.validate(GL_FRAMEBUFFER);
        frameBuffer.unbind(GL_FRAMEBUFFER);

        m.play();

        std::cout << "Start !" << std::endl;
        do
        {
            static Impulsor imp(1000);

            InputManager::ArrowsDirection directions = im.getDirectionsPressed();
            InputManager::Buttons buttons = im.getButtonsPressed();

/*
            m.getSpectrum(spectrum,512);
            g.updateSpectrum(spectrum,512);
*/

            if ( ( buttons & InputManager::INPUT_A ) == InputManager::INPUT_A )
            {
                // g.makeImpulsion(10,10,1,5);
            }

            frameBuffer.attach(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, textureBucket.getID());
            frameBuffer.bind(GL_FRAMEBUFFER);
            {
                glViewport(0, 0, e.getWindow()->getWidth(),e.getWindow()->getHeight());
                glMatrixMode(GL_PROJECTION);
                glLoadIdentity();                                                   //And reset it

                // Set orthographic projection
                gluPerspective(45.0f,1024/1024,0.1f,120.0f);
                glMatrixMode(GL_MODELVIEW);


                glClearColor(0.0f,0.0f, 0.0f, 1.0f);
                e.getWindow()->clear();

                // c.apply();

                pSimpleShader->enable();
                GLint idPulse = glGetUniformLocation(pSimpleShader->getProgram(),"amplif");
                GLUtils::checkError("glGetUniformLocation 'amplif'");
                glUniform1f(idPulse,sm.getCurrentScene()->getPulseValue());

                GLint idFade = glGetUniformLocation(pSimpleShader->getProgram(),"fade");
                GLUtils::checkError("glGetUniformLocation 'fade'");
                glUniform1f(idFade,sm.getCurrentScene()->getFadeValue());

                sm.draw(e.getWindow());

                pSimpleShader->disable();
            }
            frameBuffer.unbind(GL_FRAMEBUFFER);


            frameBuffer.attach(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, textureHBlurBucket.getID());
            frameBuffer.bind(GL_FRAMEBUFFER);
            {
                glClearColor(0.0f,0.0f, 0.25f, 1.0f);
                e.getWindow()->clear();

                pHBlurShader->enable();
                GLint idOffsetBlur = glGetUniformLocation(pHBlurShader->getProgram(),"blurOffset");
                GLUtils::checkError("glGetUniformLocation 'blurOffset'");
                /*
                glUniform1f(idOffsetBlur,hblurOffset);
                GLUtils::checkError("glUniform1f");
                */

                ScreenQuad::draw(textureBucket,*e.getWindow());
                pHBlurShader->disable();

                GLUtils::checkError("Draw");
            }
            frameBuffer.unbind(GL_FRAMEBUFFER);


            glClearColor(0.0f,0.0f, 0.0f, 1.0f);
            e.getWindow()->clear();

            pSimpleShader->enable();
            GLint idPulse = glGetUniformLocation(pSimpleShader->getProgram(),"amplif");
            GLUtils::checkError("glGetUniformLocation 'amplif'");
            glUniform1f(idPulse,sm.getCurrentScene()->getPulseValue());

            GLint idFade = glGetUniformLocation(pSimpleShader->getProgram(),"fade");
            GLUtils::checkError("glGetUniformLocation 'fade'");
            glUniform1f(idFade,sm.getCurrentScene()->getFadeValue());

            sm.draw(e.getWindow());

            pSimpleShader->disable();

            glDisable(GL_DEPTH_TEST);
            glEnable(GL_BLEND);
            glBlendFunc(GL_SRC_ALPHA,GL_ONE);

            pVBlurShader->enable();
            GLint idOffsetBlur = glGetUniformLocation(pVBlurShader->getProgram(),"blurOffset");
            GLUtils::checkError("glGetUniformLocation 'blurOffset'");

            ScreenQuad::draw(textureHBlurBucket,*e.getWindow());
            pVBlurShader->disable();

            glDisable(GL_BLEND);
            glEnable(GL_DEPTH_TEST);

            e.getWindow()->refresh();

            im.update();

            c.update();
            imp.update();

            e.waitFrame();

        } while(!im.needEscape() && !sm.update(c,&m,e.getTime()) );
        std::cout << "End !" << std::endl;

        delete pVBlurShader;
        delete pHBlurShader;
        delete pSimpleShader;

        m.stop();
    }
    catch ( EngineInitialisationException eie )
    {
        std::cerr << eie.what() << std::endl;
        return -2;
    }
    catch (GLErrorException glee )
    {
        std::cerr << glee.what() << std::endl;
        return -3;
    }
    catch (std::exception e)
    {
        std::cerr << e.what() << std::endl;
        std::cerr << "Exception cought" << std::endl;
        return -4;
    }

    return 0;
}
